@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2018 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -35,7 +35,7 @@
             ifelse(inbreeding, "without", "with"),
             " the Hardy-Weinberg equilibrium assumption)\n\n",
             "Links:    ",
-            namesof("p1", link, earg = earg, tag = FALSE), ", ", 
+            namesof("p1", link, earg = earg, tag = FALSE), ", ",
             namesof("p2", link, earg = earg, tag = FALSE),
             if (inbreeding) paste(",",
             namesof("f",  link, earg = earg, tag = FALSE)) else
@@ -46,6 +46,8 @@
          M1 = ifelse( .inbreeding , 3, 2),
          expected = TRUE,
          multipleResponses = FALSE,
+         parameters.names = c("p1", "p2",
+                if ( .inbreeding ) "f" else NULL),
          link = if ( .inbreeding )
                   c("p1" = .link , "p2" = .link , "f" = .link ) else
                   c("p1" = .link , "p2" = .link  ))
@@ -85,15 +87,15 @@
                        mustart[, 2] * mustart[, 3] +
                        mustart[, 2] * mustart[, 5] +
                        mustart[, 3] * mustart[, 5], w)
-      p1 <- if (is.numeric( .ip1 )) rep( .ip1 , len = n) else
+      p1 <- if (is.numeric( .ip1 )) rep_len( .ip1 , n) else
             weighted.mean(mustart[, 2] * mustart[, 3], w) / mydeterminant
-      p2 <- if (is.numeric( .ip2 )) rep( .ip2 , len = n) else
+      p2 <- if (is.numeric( .ip2 )) rep_len( .ip2 , n) else
             weighted.mean(mustart[, 2] * mustart[, 5], w) / mydeterminant
-      ff <- if (is.numeric( .iF  )) rep( .iF  , len = n) else
+      ff <- if (is.numeric( .iF  )) rep_len( .iF  , n) else
             weighted.mean(abs(1 - mustart[, 2] / (2 * p1 * p2)), w)
-      p1 <- rep(p1, len = n)
-      p2 <- rep(p2, len = n)
-      ff <- rep(ff, len = n)
+      p1 <- rep_len(p1, n)
+      p2 <- rep_len(p2, n)
+      ff <- rep_len(ff, n)
       p1[p1 < 0.05] <- 0.05
       p1[p1 > 0.99] <- 0.99
       p2[p2 < 0.05] <- 0.05
@@ -146,6 +148,20 @@
                        log = TRUE, dochecking = FALSE))
       },
   vfamily = c("A1A2A3", "vgenetic"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    p1 <- eta2theta(eta[, 1], link = .link , earg = .earg )
+    p2 <- eta2theta(eta[, 2], link = .link , earg = .earg )
+    p3 <- 1-p1-p2
+    okay1 <- all(is.finite(p1)) && all(0 < p1 & p1 < 1) &&
+             all(is.finite(p2)) && all(0 < p2 & p2 < 1) &&
+             all(is.finite(p3)) && all(0 < p3 & p3 < 1)
+    okay2 <- TRUE
+    if ( .inbreeding ) {
+      f <- eta2theta(eta[, 3], link = .link , earg = .earg )
+      okay2 <- all(is.finite(f)) && all(0 <= f)  # && all(f < 1)
+    }
+    okay1 && okay2
+  }, list( .link = link, .earg = earg, .inbreeding = inbreeding) )),
   deriv = eval(substitute(expression({
     p1 <- eta2theta(eta[, 1], link = .link , earg = .earg )
     p2 <- eta2theta(eta[, 2], link = .link , earg = .earg )
@@ -157,7 +173,7 @@
                   0, -2*(1-f)*p2, -f - 2*p3*(1-f))
       dP2 <- cbind(0, 2*p1*(1-f), -2*(1-f)*p1, f+2*p2*(1-f),
                    2*(1-f)*(1-p1-2*p2), -f - 2*p3*(1-f))
-      dP3 <- cbind(p1*(1-p1), -2*p1*p2, -2*p1*p3, p2*(1-p2), -2*p2*p3, 
+      dP3 <- cbind(p1*(1-p1), -2*p1*p2, -2*p1*p3, p2*(1-p2), -2*p2*p3,
                    p3*(1-p3))
       dl1 <- rowSums(y * dP1 / mu)
       dl2 <- rowSums(y * dP2 / mu)
@@ -165,7 +181,7 @@
       dPP.deta <- dtheta.deta(cbind(p1, p2, f),
                               link = .link , earg = .earg )
       c(w) * cbind(dPP.deta[, 1] * dl1,
-                   dPP.deta[, 2] * dl2, 
+                   dPP.deta[, 2] * dl2,
                    dPP.deta[, 3] * dl3)
     } else {
       dl.dp1 <- (2*y[, 1]+y[, 2]+y[, 4])/p1 -
@@ -183,7 +199,7 @@
   weight = eval(substitute(expression({
     if ( .inbreeding ) {
       dPP <- array(c(dP1, dP2, dP3), c(n, 6, 3))
-      wz <- matrix(as.numeric(NA), n, dimm(M))  # dimm(M)==6 because M==3
+      wz <- matrix(NA_real_, n, dimm(M))  # dimm(M)==6 because M==3
       for (i1 in 1:M)
         for (i2 in i1:M) {
           index <- iam(i1, i2, M)
@@ -193,7 +209,7 @@
       }
     } else {
       qq <- 1-p1-p2
-      wz <- matrix(as.numeric(NA), n, dimm(M))  # dimm(M)==3 because M==2
+      wz <- matrix(NA_real_, n, dimm(M))  # dimm(M)==3 because M==2
       ned2l.dp12  <-  2 * (1/p1 + 1/qq)
       ned2l.dp22  <-  2 * (1/p2 + 1/qq)
       ned2l.dp1dp2 <-  2 / qq
@@ -223,8 +239,8 @@
   new("vglmff",
   blurb = c("MNSs Blood Group System (MS-Ms-MNS-MNs-NS-Ns phenotype)\n\n",
             "Links:    ",
-            namesof("mS", link, earg = earg), ", ", 
-            namesof("ms", link, earg = earg), ", ", 
+            namesof("mS", link, earg = earg), ", ",
+            namesof("ms", link, earg = earg), ", ",
             namesof("nS", link, earg = earg, tag = FALSE)),
   deviance = Deviance.categorical.data.vgam,
   initialize = eval(substitute(expression({
@@ -251,19 +267,20 @@
          namesof("nS", .link , earg = .earg , tag = FALSE))
 
     if (is.null(etastart)) {
-      ms <- if (is.numeric(.ims)) rep(.ims, n) else
+      ms <- if (is.numeric(.ims)) rep_len( .ims , n) else
                  c(sqrt(mustart[, 2]))
-      ns <- c(sqrt(mustart[,6]))
-      nS <- if (is.numeric(.inS)) rep(.inS, n) else
-          c(-ns + sqrt(ns^2 + mustart[,5]))  # Solve a quadratic eqn
-      mS <- if (is.numeric(.imS)) rep(.imS, n) else
+      ns <- c(sqrt(mustart[, 6]))
+      nS <- if (is.numeric(.inS)) rep_len( .inS , n) else
+          c(-ns + sqrt(ns^2 + mustart[, 5]))  # Solve a quadratic eqn
+      mS <- if (is.numeric(.imS)) rep_len( .imS , n) else
               1-ns-ms-nS
       etastart <- cbind(theta2eta(mS, .link , earg = .earg ),
                        theta2eta(ms, .link , earg = .earg ),
                        theta2eta(nS, .link , earg = .earg ))
       mustart <- NULL  # Since etastart has been computed.
     }
-  }), list( .link = link, .imS = imS, .ims = ims, .inS = inS, .earg = earg))),
+  }), list( .link = link, .imS = imS, .ims = ims, .inS = inS,
+            .earg = earg))),
   linkinv = eval(substitute(function(eta, extra = NULL) {
     mS <- eta2theta(eta[, 1], link = .link , earg = .earg )
     ms <- eta2theta(eta[, 2], link = .link , earg = .earg )
@@ -293,6 +310,17 @@
                        log = TRUE, dochecking = FALSE))
     },
   vfamily = c("MNSs", "vgenetic"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    mS <- eta2theta(eta[, 1], link = .link , earg = .earg )
+    ms <- eta2theta(eta[, 2], link = .link , earg = .earg )
+    nS <- eta2theta(eta[, 3], link = .link , earg = .earg )
+    ns <- 1-mS-ms-nS
+    okay1 <- all(is.finite(mS)) && all(0 < mS & mS < 1) &&
+             all(is.finite(ms)) && all(0 < ms & ms < 1) &&
+             all(is.finite(nS)) && all(0 < nS & nS < 1) &&
+             all(is.finite(ns)) && all(0 < ns & ns < 1)
+    okay1
+  }, list( .link = link, .earg = earg) )),
   deriv = eval(substitute(expression({
     mS <- eta2theta(eta[, 1], link = .link , earg = .earg )
     ms <- eta2theta(eta[, 2], link = .link , earg = .earg )
@@ -304,12 +332,13 @@
     dl1 <- rowSums(y * dP1 / mu)
     dl2 <- rowSums(y * dP2 / mu)
     dl3 <- rowSums(y * dP3 / mu)
-    dPP.deta <- dtheta.deta(cbind(mS, ms, nS), link = .link , earg = .earg )
+    dPP.deta <- dtheta.deta(cbind(mS, ms, nS),
+                            link = .link , earg = .earg )
     c(w) * dPP.deta * cbind(dl1, dl2, dl3)
   }), list( .link = link, .earg = earg))),
   weight = eval(substitute(expression({
     dPP <- array(c(dP1,dP2,dP3), c(n,6, 3))
-    wz <- matrix(as.numeric(NA), n, dimm(M))  # dimm(M)==6 because M==3
+    wz <- matrix(NA_real_, n, dimm(M))  # dimm(M)==6 because M==3
     for (i1 in 1:M)
       for (i2 in i1:M) {
         index <- iam(i1,i2, M)
@@ -341,13 +370,21 @@
   new("vglmff",
   blurb = c("ABO Blood Group System (A-B-AB-O phenotype)\n\n",
             "Links:    ",
-            namesof("pA", link.pA, earg = earg.pA, tag = FALSE), ", ", 
+            namesof("pA", link.pA, earg = earg.pA, tag = FALSE), ", ",
             namesof("pB", link.pB, earg = earg.pB, tag = FALSE)),
   deviance = Deviance.categorical.data.vgam,
+
+  constraints = eval(substitute(expression({
+    constraints <- cm.zero.VGAM(constraints, x = x, .zero , M = M,
+                                predictors.names = predictors.names,
+                                M1 = 2)
+  }), list( .zero = zero ))),
+
   infos = eval(substitute(function(...) {
     list(M1 = 2,
          Q1 = 4,
          multipleResponses = FALSE,
+         parameters.names = c("pA", "pB"),
          expected = TRUE,
          zero = .zero ,
          link = c("pA" = .link.pA , "pB" = .link.pB ),
@@ -357,11 +394,6 @@
            .earg.pA = earg.pA, .earg.pB = earg.pB,
            .zero = zero ))),
 
-  constraints = eval(substitute(expression({
-    constraints <- cm.zero.VGAM(constraints, x = x, .zero , M = M)
-  }), list( .zero = zero ))),
-
-
   initialize = eval(substitute(expression({
     mustart.orig <- mustart
 
@@ -376,7 +408,7 @@
         setequal(ok.col.ny, col.ny)) {
       if (!all(ok.col.ny == col.ny))
         stop("the columns of the response matrix should have names ",
-             "(output of colnames()) ordered as c('A','B','AB','O')")
+             "(output of colnames()) ordered as c('A', 'B', 'AB', 'O')")
     }
 
 
@@ -386,12 +418,12 @@
     mustart <- (y + mustart) / 2
 
     if (!length(etastart)) {
-      pO <- if (is.Numeric( .ipO )) rep( .ipO , len = n) else
-        rep(c(sqrt( weighted.mean(mustart[, 4], w)) ), len = n)
-      pA <- if (is.Numeric( .ipA )) rep( .ipA , len = n) else
-        rep(c(1 - sqrt(weighted.mean(mustart[, 2] + mustart[, 4], w))),
-            len = n)
-      pB <- if (is.Numeric( .ipB )) rep( .ipB , len = n) else
+      pO <- if (is.Numeric( .ipO )) rep_len( .ipO , n) else
+        rep_len(c(sqrt( weighted.mean(mustart[, 4], w)) ), n)
+      pA <- if (is.Numeric( .ipA )) rep_len( .ipA , n) else
+              rep_len(c(1 - sqrt(weighted.mean(mustart[, 2] +
+                                               mustart[, 4], w))), n)
+      pB <- if (is.Numeric( .ipB )) rep_len( .ipB , n) else
             abs(1 - pA - pO)
       etastart <- cbind(theta2eta(pA, .link.pA , earg = .earg.pA ),
                         theta2eta(pB, .link.pB , earg = .earg.pB ))
@@ -409,7 +441,7 @@
       cbind(A  = pA*(pA+2*pO),
             B  = pB*(pB+2*pO),
             AB = 2*pA*pB,
-            O  = pO*pO) 
+            O  = pO*pO)
   }, list( .link.pA = link.pA, .link.pB = link.pB,
            .earg.pA = earg.pA, .earg.pB = earg.pB ))),
 
@@ -430,11 +462,21 @@
     },
 
   vfamily = c("ABO", "vgenetic"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    ppp <- eta2theta(eta[, 1], link = .link.pA , earg = .earg.pA )
+    qqq <- eta2theta(eta[, 2], link = .link.pB , earg = .earg.pB )
+    rrr <- 1 - ppp - qqq  # abs(1 - ppp - qqq) prior to 20160624
+    okay1 <- all(is.finite(ppp)) && all(0 < ppp & ppp < 1) &&
+             all(is.finite(qqq)) && all(0 < qqq & qqq < 1) &&
+             all(is.finite(rrr)) && all(0 < rrr & rrr < 1)
+    okay1
+  }, list( .link.pA = link.pA, .link.pB = link.pB,
+           .earg.pA = earg.pA, .earg.pB = earg.pB ))),
 
   deriv = eval(substitute(expression({
     ppp <- eta2theta(eta[, 1], link = .link.pA , earg = .earg.pA )
     qqq <- eta2theta(eta[, 2], link = .link.pB , earg = .earg.pB )
-    rrr <- abs(1 - ppp - qqq)
+    rrr <- 1 - ppp - qqq  # abs(1 - ppp - qqq)
 
 
     pbar <- 2*rrr + ppp
@@ -455,7 +497,7 @@
             .earg.pA = earg.pA, .earg.pB = earg.pB ))),
 
   weight = eval(substitute(expression({
-    wz <- matrix(as.numeric(NA), n, dimm(M))  # dimm(M)==3 because M==2
+    wz <- matrix(NA_real_, n, dimm(M))  # dimm(M)==3 because M==2
 
     ned2l.dp2  <- (1 + 2/ppp + 4*qqq/qbar + ppp/pbar)
     ned2l.dq2  <- (1 + 2/qqq + 4*ppp/pbar + qqq/qbar)
@@ -505,8 +547,8 @@
     mustart <- (y + mustart) / 2
 
     if (is.null(etastart)) {
-      p.init <- if (is.numeric( .init.p )) rep( .init.p , len = n) else
-        rep(c(sqrt(4 * weighted.mean(mustart[, 4], w))), len = n)
+      p.init <- if (is.numeric( .init.p )) rep_len( .init.p , n) else
+        rep_len(c(sqrt(4 * weighted.mean(mustart[, 4], w))), n)
 
       etastart <- cbind(theta2eta(p.init, .link , earg = .earg ))
       etastart <- jitter(etastart)
@@ -519,7 +561,7 @@
     cbind(AB = 0.5 + pp4,
           Ab = 0.25 - pp4,
           aB = 0.25 - pp4,
-          ab = pp4) 
+          ab = pp4)
   }, list( .link = link, .earg = earg))),
 
   last = eval(substitute(expression({
@@ -538,6 +580,11 @@
                        log = TRUE, dochecking = FALSE))
     },
   vfamily = c("AB.Ab.aB.ab", "vgenetic"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    pp <- eta2theta(eta, link = .link , earg = .earg )
+    okay1 <- all(is.finite(pp)) && all(0 < pp & pp < 1)
+    okay1
+  }, list( .link = link, .earg = earg ))),
   deriv = eval(substitute(expression({
     pp <- eta2theta(eta, link = .link , earg = .earg )
 
@@ -579,7 +626,7 @@
            ipA = NULL,
            ifp = NULL,
            zero = NULL) {
-    
+
   linkp <- as.list(substitute(linkp))
   eargp <- link2list(linkp)
   linkp <- attr(eargp, "function.name")
@@ -591,7 +638,7 @@
   if (!is.logical(inbreeding) || length(inbreeding) > 1)
     stop("argument 'inbreeding' must be a single logical")
 
-  
+
 
   new("vglmff",
   blurb = c("AA-Aa-aa phenotype (",
@@ -607,10 +654,12 @@
     list(M1 = ifelse( .inbreeding , 2, 1),
          Q1 = 3,
          multipleResponses = FALSE,
+         parameters.names = c("pA",
+                if ( .inbreeding ) "f" else NULL),
          expected = TRUE,
          zero = .zero ,
          link = if ( .inbreeding ) c("pA" = .linkp , "f" = .linkf ) else
-                            c("pA" = .linkp ))
+                                   c("pA" = .linkp ))
   }, list( .linkp = linkp,
            .linkf = linkf, .inbreeding = inbreeding,
            .zero = zero ))),
@@ -638,12 +687,12 @@
         if ( .inbreeding )
         namesof("f",  .linkf , earg = .eargf , tag = FALSE) else NULL)
     mustart <- (y + mustart) / 2
-        
+
 
     if (is.null(etastart)) {
-      pA <- if (is.numeric( .ipA )) rep( .ipA , len = n) else
-              rep(c(sqrt( weighted.mean(mustart[, 1], w))), len = n)
-      fp <- if (is.numeric( .ifp )) rep( .ifp , len = n) else
+      pA <- if (is.numeric( .ipA )) rep_len( .ipA , n) else
+              rep_len(c(sqrt( weighted.mean(mustart[, 1], w))), n)
+      fp <- if (is.numeric( .ifp )) rep_len( .ifp , n) else
               runif(n)  # 1- mustart[, 2]/(2*pA*(1-pA))
       etastart <- cbind(theta2eta(pA, .linkp , earg = .eargp ),
                         if ( .inbreeding )
@@ -688,6 +737,19 @@
                          log = TRUE, dochecking = FALSE))
     },
   vfamily = c("AA.Aa.aa", "vgenetic"),
+  validparams = eval(substitute(function(eta, y, extra = NULL) {
+    eta <- as.matrix(eta)
+    pA <- eta2theta(eta[, 1], link = .linkp , earg = .eargp )
+    okay1 <- all(is.finite(pA)) && all(0 < pA & pA < 1)
+    okay2 <- TRUE
+    if ( .inbreeding ) {
+      fp <- eta2theta(eta[, 2], link = .linkf , earg = .eargf )
+      okay2 <- all(is.finite(fp)) && all(0 <= fp)  # && all(fp < 1)
+    }
+    okay1 && okay2
+  }, list( .linkp = linkp, .linkf = linkf,
+           .eargp = eargp, .eargf = eargf,
+           .inbreeding = inbreeding ))),
   deriv = eval(substitute(expression({
     eta <- as.matrix(eta)
     pA <- eta2theta(eta[, 1], link = .linkp , earg = .eargp )
@@ -708,7 +770,7 @@
       dfp.deta <- dtheta.deta(fp, link = .linkf , earg = .eargf )
 
       c(w) * cbind(dPP.deta * dl1,
-                   dfp.deta * dl2)      
+                   dfp.deta * dl2)
     } else {
       nAA <- c(w) * y[, 1]
       nAa <- c(w) * y[, 2]
@@ -716,7 +778,7 @@
       dl.dpA <- (2*nAA+nAa)/pA - (nAa+2*naa)/(1-pA)
       dpA.deta <- dtheta.deta(pA, link = .linkp , earg = .eargp )
       dl.dpA * dpA.deta
-    }  
+    }
   }), list( .linkp = linkp, .linkf = linkf,
             .eargp = eargp, .eargf = eargf,
             .inbreeding = inbreeding ))),
@@ -725,7 +787,7 @@
       dPP <- array(c(dP1, dP2), c(n, 3, 2))
       dPP.deta <- cbind(dtheta.deta(pA, link = .linkp , earg = .eargp ),
                         dtheta.deta(fp, link = .linkf , earg = .eargf ))
-      wz <- matrix(as.numeric(NA), n, dimm(M))  # dimm(M)==3 because M==2
+      wz <- matrix(NA_real_, n, dimm(M))  # dimm(M)==3 because M==2
       for (i1 in 1:M)
         for (i2 in i1:M) {
           index <- iam(i1, i2, M)
