@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2017 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2018 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -55,7 +55,8 @@
 
 
   if (Rank == 0 && !has.intercept)
-    warning("probably 'has.intercept == TRUE' is better for a rank-0 model")
+    warning("probably 'has.intercept == TRUE' is better for ",
+            "a rank-0 model")
 
 
 
@@ -119,29 +120,34 @@
 
 
   .rcim.df <-
-    if (!noroweffects) data.frame("Row.2" = I.col(2, nrow(y))) else  # See below
-    if (!nocoleffects) data.frame("Col.2" = I.col(2, nrow(y))) else  # See below
-    stop("at least one of 'noroweffects' and 'nocoleffects' must be FALSE")
+    if (!noroweffects)
+      data.frame("Row.2" = I.col(2, nrow(y))) else  # See below
+    if (!nocoleffects)
+      data.frame("Col.2" = I.col(2, nrow(y))) else  # See below
+      stop("at least one of 'noroweffects' and 'nocoleffects' ",
+           "must be FALSE")
 
 
-  min.row.val <- rindex[1]  # == min(rindex) since it is sorted # Usually 2
-  min.col.val <- cindex[1]  # == min(cindex) since it is sorted # Usually 2
+  min.row.val <- rindex[1]  # == min(rindex) since its sorted # Usually 2
+  min.col.val <- cindex[1]  # == min(cindex) since its sorted # Usually 2
   if (!noroweffects) {
-    colnames( .rcim.df ) <- paste(rprefix, as.character(min.row.val),  # "2",
-                                  sep = "")  # Overwrite "Row.2"
+    colnames( .rcim.df ) <-
+      paste(rprefix, as.character(min.row.val),  # "2",
+            sep = "")  # Overwrite "Row.2"
   } else if (!nocoleffects) {
-    colnames( .rcim.df ) <- paste(cprefix, as.character(min.col.val),  # "2",
-                                  sep = "")  # Overwrite "Col.2"
+    colnames( .rcim.df ) <-
+      paste(cprefix, as.character(min.col.val),  # "2",
+            sep = "")  # Overwrite "Col.2"
   }
 
 
 
   yn1 <- if (length(dimnames(y)[[1]])) dimnames(y)[[1]] else
-             paste(iprefix, 1:nrow(y), sep = "")
+             param.names(iprefix, nrow(y))
   warn.save <- options()$warn
   options(warn = -3)  # Suppress the warnings (hopefully, temporarily)
   if (any(!is.na(as.numeric(substring(yn1, 1, 1)))))
-    yn1 <- paste(iprefix, 1:nrow(y), sep = "")
+    yn1 <- param.names(iprefix, nrow(y))
   options(warn = warn.save)
 
 
@@ -387,13 +393,13 @@ setMethod("summary", "rcim",
   CCC <- dim(mat)[2]
 
   rnames <- if (is.null(rownames(mat))) {
-    paste("X", 1:RRR, sep = "")
+    param.names("X", RRR)
   } else {
     rownames(mat)
   }
 
   cnames <- if (is.null(colnames(mat))) {
-    paste("Y", 1:CCC, sep = "")
+    param.names("Y", CCC)
   } else {
     colnames(mat)
   }
@@ -602,15 +608,15 @@ setMethod("plot", "rcim",
   ind1 <- if (is.character(roffset))
              which(rownames(mat) == roffset) else
                    if (is.numeric(roffset)) roffset + 1 else
-                     stop("argument 'roffset' not matched (character). ",
-                           "It must be numeric, ",
+                     stop("argument 'roffset' not matched (character).",
+                           " It must be numeric, ",
                            "else character and match the ",
                            "row names of the response")
   ind2 <- if (is.character(coffset))
              which(colnames(mat) == coffset) else
                    if (is.numeric(coffset)) coffset + 1 else
-                     stop("argument 'coffset' not matched (character). ",
-                           "It must be numeric, ",
+                     stop("argument 'coffset' not matched (character).",
+                           " It must be numeric, ",
                            "else character and match the ",
                            "column names of the response")
 
@@ -633,11 +639,11 @@ setMethod("plot", "rcim",
 
   rownames.mat <- rownames(mat)
   if (length(rownames.mat) != nrow(mat))
-    rownames.mat <- paste(rprefix, 1:nrow(mat), sep = "")
+    rownames.mat <- param.names(rprefix, nrow(mat))
 
   colnames.mat <- colnames(mat)
   if (length(colnames.mat) != ncol(mat))
-    colnames.mat <- paste(cprefix, 1:ncol(mat), sep = "")
+    colnames.mat <- param.names(cprefix, ncol(mat))
 
 
   newrn <- if (roffset > 0)
@@ -753,7 +759,7 @@ Confint.nb1 <- function(nb1, level = 0.95) {
 
 
   myvec <- cbind(c(-1, 1, rep_len(0, nrow(myvcov) - 2)))
-  (se.mydiff <- sqrt(t(myvec) %*%  myvcov %*%  myvec))
+  se.mydiff <- c(sqrt(t(myvec) %*% myvcov %*% myvec))  # c() to drop the 2-D
 
   ci.mydiff <- mydiff + c(-1, 1) * qnorm(1 - (1 - level)/2) * se.mydiff
 
@@ -1155,7 +1161,7 @@ summary.qvar <- function(object, ...) {
       is.matrix(object@extra$attributes.y$estimates))
     names( estimates) <- rownames(object@extra$attributes.y$estimates)
   if (!length(names(estimates)))
-    names( estimates) <- paste("Level", seq_along(estimates), sep = "")
+    names( estimates) <- param.names("Level", length(estimates))
 
 
   regularVar <- c(object@extra$attributes.y$regularVar)
@@ -1285,8 +1291,7 @@ qvplot   <-  function(object,
   if (length(level.names) == length(estimates)) {
     names(estimates) <- level.names
   } else if (!length(names(estimates)))
-    names(estimates) <- paste("Level", seq_along(estimates),
-                              sep = "")
+    names(estimates) <- param.names("Level", length(estimates))
 
 
 
